unit TomasGraphicsForm;

(***********************************************************************)
(*  This software is copyleft software;                                *)
(*  You may distribute or change the software as you wish, but         *)
(*  this header must always be present.                                *)
(*  No one is allowed to claim copyright on this source code.          *)
(*  All precompiled software using this unit should be accompanied     *)
(*  by this source code.                                               *)
(*                                                                     *)
(*  Original contents on http://home.planet.nl/~veekehpm               *)
(*  Contact: Hans P.M. Veeke         H.P.M.Veeke@wbmt.tudelft.nl       *)
(***********************************************************************)

(***********************************************************************)
(* Update history                                                      *)
(* 24-11-2001: LoadSettings corrected; FigureCount correctly set       *)
(*             CollectionsGrid made resizable                          *)                                      
(* 20-06-2000: CSV file added to load and save settings                *)
(*             tcs files before this date won't work anymore!!!        *)
(***********************************************************************)




interface

uses
  Tomas,
  TomasCollectionForm,
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  Buttons,
  ToolWin,
  ComCtrls,
  Grids, ExtCtrls;

type
  TGraphicsForm = class(TForm)
    CollectionsGrid: TStringGrid;
    SelectionCombo: TComboBox;
    ColorDialog1: TColorDialog;
    SaveSettingsButton: TBitBtn;
    SaveSettingsDialog: TSaveDialog;
    LoadSettingsDialog: TOpenDialog;
    LoadSettingsButton: TBitBtn;
    CsvEdit: TEdit;
    procedure GoResizeGrid(Sender: TObject);
    procedure GoCreateGraphicsForm(Sender: TObject);
    procedure GoHandleClick(Sender: TObject);
    procedure GoHandleComboSelection(Sender: TObject);
    procedure GoDrawCells(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure GoSaveSettings(Sender: TObject);
    procedure GoLoadSettings(Sender: TObject);
    procedure CheckFormClosing(Sender: TObject; var Action: TCloseAction);
    procedure GoHandleCSVEdit(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    FigureCount: Integer;
  public
    { Public declarations }
    Procedure LoadSettings(FName: String);
  end;

TomasCollection = class(TomasCollectionData)
  Private
    FName: String;
    FRow: Integer;
  Published
    Constructor Create(CName: String);
    Destructor Destroy; Override;
    Property CTitle: String read FName;
  End;

var
  GraphicsForm: TGraphicsForm;

implementation

{$R *.DFM}

Var
  CollectionRoom: TList;
  FigureRoom: TList;

Constructor TomasCollection.Create(CName: String);
  Begin
  Inherited Create;
  FName:=CName;
  CollectionRoom.Add(Self);
  With GraphicsForm.CollectionsGrid Do
    Begin
    FRow:=RowCount-1;
    Cells[0,FRow]:=FName;
    Cells[1,FRow]:='Hidden';
    Cells[2,FRow]:='';
    Cells[3,FRow]:='';
    RowCount:=RowCount + 1;
    End;
  End;  {TomasCollection.Create}

Destructor TomasCollection.Destroy;
  Var
    i: integer;
  Begin
  CollectionRoom.Remove(Self);
  With GraphicsForm.CollectionsGrid Do
    Begin
    For i:=FRow + 1 To RowCount - 2 Do
      Begin
      Cells[0,i-1]:=Cells[0,i];
      Cells[1,i-1]:=Cells[1,i];
      Cells[2,i-1]:=Cells[2,i];
      Cells[3,i-1]:=Cells[3,i];
      Cells[4,i-1]:=Cells[4,i];
      Cells[5,i-1]:=Cells[5,i];
      Cells[6,i-1]:=Cells[6,i];
      Cells[7,i-1]:=Cells[7,i];
      End;
    If RowCount > 2 Then
      Begin
      RowCount:=RowCount - 1;
      End;
    End;
  Inherited Destroy;
  End;  {TomasCollection.Destroy}

procedure TGraphicsForm.GoCreateGraphicsForm(Sender: TObject);
begin
CollectionRoom:=TList.Create;
FigureRoom:=TList.Create;
With CollectionsGrid Do
  Begin
  RowCount:=2;
  ColWidths[0]:=30 * 9;
  ColWidths[1]:=10 * 9;
  ColWidths[2]:=10 * 9;
  ColWidths[3]:=20 * 9;
  Cells[0,0]:='Title';
  Cells[1,0]:='Form';
  Cells[2,0]:='Color';
  Cells[3,0]:='CSV';
  End;
ClientWidth:=70 * 9 + 15;
CollectionsGrid.Width:=70 * 9 + 12 ;
Left:=0;
end;

procedure TGraphicsForm.GoHandleClick(Sender: TObject);
Var
  Ix: Integer;
  ClickCollection: TomasCollection;
  Figure: TomasCollectionFigure;
  Rect: TRect;

begin
With GraphicsForm.CollectionsGrid Do
  Begin
  If (Row <= 0) Or (Col<=0) Or (Row >= RowCount - 1) Then
    Begin
    Exit;
    End;
  ClickCollection:=CollectionRoom.Items[Row-1];
  Case Col Of
    1:Begin    {form-field}
      SelectionCombo.Clear;
      If ClickCollection.Figure<>Nil Then
        Begin
        SelectionCombo.Text:=ClickCollection.Figure.Title;
        End
      Else
        Begin
        SelectionCombo.Text:='Hidden';
        End;
      SelectionCombo.Items.Add('Hidden');
      SelectionCombo.Items.Add('New Figure');
      For Ix:=0 To FigureRoom.Count - 1 Do
        Begin
        Figure:=FigureRoom.Items[Ix];
        SelectionCombo.Items.Add(Figure.Title);
        End;
      With GraphicsForm.CollectionsGrid Do
        Begin
        Rect:=CellRect(Col,Row);
        End;
      SelectionCombo.Top:=GraphicsForm.CollectionsGrid.Top + Rect.Top;
      SelectionCombo.Left:=GraphicsForm.Collectionsgrid.Left + Rect.Left;
      SelectionCombo.Width:=Rect.Right - Rect.Left;
      SelectionCombo.Show;
      SelectionCombo.BringToFront;
      SelectionCombo.SetFocus;
      End;
    2:Begin    {color}
      ColorDialog1.Color:=ClickCollection.CColor;
      If ColorDialog1.Execute Then
        Begin
        ClickCollection.CColor:=ColorDialog1.Color;
        ClickCollection.RefreshData;
        End;
      End;
    3:Begin    {CSV}
      CSVEdit.Text:=ClickCollection.CSVFile;
      With GraphicsForm.CollectionsGrid Do
        Begin
        Rect:=CellRect(Col,Row);
        End;
      CSVEdit.Top:=GraphicsForm.CollectionsGrid.Top + Rect.Top;
      CSVEdit.Left:=GraphicsForm.Collectionsgrid.Left + Rect.Left;
      CSVEdit.Width:=Rect.Right - Rect.Left;
      CSVEdit.Show;
      CSVEdit.BringToFront;
      CSVEdit.SetFocus;
      End;
    End;
  End;
end;

Procedure InitializeGraphics;
  Begin
  Application.CreateForm(TGraphicsForm,GraphicsForm);
  TheGraphicsForm:=GraphicsForm;
  GraphicsForm.Show;
  GraphicsForm.FigureCount:=0;
  End;

procedure TGraphicsForm.GoHandleComboSelection(Sender: TObject);
Var
  ClickCollection: TomasCollection;
  S: String;
  TheFigure: TomasCollectionFigure;
  OldFigure: TomasCollectionFigure;
begin
With CollectionsGrid Do
  Begin
  ClickCollection:=CollectionRoom.Items[Row-1];
  Case CollectionsGrid.Col Of
    1:Begin    {form select for collection}
      S:=Trim(SelectionCombo.Text);
      If (S = 'Hidden') Or (S = 'New Figure') Then
        Begin
        If Assigned(ClickCollection.Figure) Then
          Begin
          TheFigure:=ClickCollection.Figure;
          TheFigure.RemoveCollection(ClickCollection);
          If TheFigure.NrOfCollections = 0 Then
            Begin
            FigureRoom.Remove(TheFigure);
            TheFigure.Free;
            End;
          End;
        If S = 'New Figure' Then
          Begin
          Inc(FigureCount);
          TheFigure:=TomasCollectionFigure.Create('Figure ' + IntToStr(FigureCount));
          FigureRoom.Add(TheFigure);
          TheFigure.AddCollection(ClickCollection,ClickCollection.CTitle);
          S:=TheFigure.Title;
          End;
        End
      Else
        Begin
        If Assigned(ClickCollection.Figure) Then
          Begin
          If S <> Trim(ClickCollection.Figure.Title) Then
            Begin
            OldFigure:=ClickCollection.Figure;
            OldFigure.RemoveCollection(ClickCollection);
            TheFigure:=FigureRoom.Items[SelectionCombo.ItemIndex-2];
            TheFigure.AddCollection(ClickCollection,ClickCollection.CTitle);
            If OldFigure.NrOfCollections = 0 Then
              Begin
              FigureRoom.Remove(OldFigure);
              OldFigure.Free;
              End;
            End;
          End
        Else
          Begin
          TheFigure:=FigureRoom.Items[SelectionCombo.ItemIndex-2];
          TheFigure.AddCollection(ClickCollection,ClickCollection.CTitle);
          End;
        S:=ClickCollection.Figure.Title;
        End;
      SelectionCombo.Hide;
      Cells[1,Row]:=S;
      End;
    End;
  End;
end;

procedure TGraphicsForm.GoDrawCells(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
Var
  CellCollection: TomasCollection;
begin
with Sender as TStringGrid do
  begin
  If (ACol=2) And (ARow > 0) And (ARow < CollectionsGrid.RowCount - 1) Then
    Begin
    CellCollection:=CollectionRoom.Items[ARow - 1];
    Canvas.Brush.Color := CellCollection.CColor;
    Canvas.FillRect(Rect);
    End;
  End;
end;

procedure TGraphicsForm.GoSaveSettings(Sender: TObject);
Var
  SaveFile: textFile;
  Collection: TomasCollection;
  Figure: TomasCollectionFigure;
  i: integer;
begin
If SaveSettingsDialog.Execute Then
  Begin
  AssignFile(SaveFile,SaveSettingsDialog.FileName);
  Rewrite(SaveFile);
  WriteLn(SaveFile,'[Figures]');
  For i:=1 To FigureRoom.Count Do
    Begin
    Figure:=FigureRoom.Items[i-1];
    Figure.SaveToSettingsFile(SaveFile);
    End;
  WriteLn(SaveFile,'[Collections]');
  For i:=1 To CollectionRoom.Count Do
    begin
    Collection:=CollectionRoom.Items[i-1];
    WriteLn(SaveFile,'Title=' + Collection.CTitle);
    Write(SaveFile,'Figure=');
    If Assigned(Collection.Figure) Then
      begin
      Write(SaveFile,Collection.Figure.Title);
      End;
    WriteLn(SaveFile);
    WriteLn(SaveFile,'Color='+IntToStr(Collection.CColor));
    WriteLn(SaveFile,'CSV=' + Collection.CSVFile);
    End;
  CloseFile(SaveFile);
  End;
end;

Procedure TGraphicsForm.LoadSettings(FName: String);
  Var
    Figure: TomasCollectionFigure;
    Collection: TomasCollection;
    LoadFile: TextFile;
    S: String;
    I: Integer;
    Found: Boolean;
  Begin
  If FileExists(FName) Then
    Begin
    AssignFile(LoadFile,FName);
    Reset(LoadFile);
    ReadLn(LoadFile,S);
    if S<>'[Figures]' Then
      Begin
      MessageDlg('Invalid settings format',mtError,[mbOK],0);
      Exit;
      End;
    {empty current figureroom}
    While FigureRoom.Count > 0 Do
      Begin
      Figure:=FigureRoom.Items[0];
      FigureRoom.Remove(Figure);
      Figure.Free;
      End;
    ReadLn(LoadFile,S);
    While S<>'[Collections]' Do
      Begin
      Figure:=TomasCollectionFigure.Create(Copy(S,Pos('=',S) + 1,Length(S) - Pos('=',S)));
      FigureRoom.Add(Figure);
      Inc(FigureCount);
      Figure.ReadFromSettingsFile(LoadFile);
      ReadLn(LoadFile,S);
      End;
    While NOT(EOF(LoadFile)) Do
      Begin
      ReadLn(LoadFile,S);
      S:=Trim(Copy(S,Pos('=',S)+1,Length(S)-Pos('=',S)));
      I:=0;
      Found:=FALSE;
      While (i < CollectionRoom.Count) And NOT(Found) Do
        begin
        Collection:=CollectionRoom.Items[i];
        If Collection.CTitle=S Then
          begin
          Found:=TRUE;
          End
        Else
          Begin
          Inc(i);
          End;
        End;
      If Found Then
        Begin
        ReadLn(LoadFile,S);
        S:=Trim(Copy(S,Pos('=',S)+1,Length(S)-Pos('=',S)));
        I:=0;
        Found:=FALSE;
        While (I<FigureRoom.Count) And NOT(Found) Do
          begin
          Figure:=FigureRoom.Items[i];
          If Figure.Title=S Then
            begin
            Found:=TRUE;
            End
          Else
            Begin
            Inc(i);
            End;
          End;
        If Found Then
          Begin
          Figure.AddCollection(Collection,Collection.CTitle);
          CollectionsGrid.Cells[1,Collection.FRow]:=Figure.Title;
          End;
        ReadLn(LoadFile,S);
        S:=Trim(Copy(S,Pos('=',S)+1,Length(S)-Pos('=',S)));
        Collection.CColor:=TColor(StrToInt(S));
        ReadLn(LoadFile,S);
        Collection.SetCSVFile(Trim(Copy(S,Pos('=',S)+1,Length(S)-Pos('=',S))));
        End
      Else
        Begin
        ReadLn(LoadFile,S);
        ReadLn(LoadFile,S);
        ReadLn(LoadFile,S);
        End;
      End;
    CloseFile(LoadFile);
    End
  Else
    Begin
    MessageDlg('Settings file ' + FName + ' not found!',mtWarning,[mbOK],0);
    End;
  End;

procedure TGraphicsForm.GoLoadSettings(Sender: TObject);
begin
If LoadSettingsDialog.Execute Then
  Begin
  LoadSettings(LoadSettingsDialog.FileName);
  End;
end;

procedure TGraphicsForm.CheckFormClosing(Sender: TObject;
  var Action: TCloseAction);
Var
  Collection: TomasCollection;
begin
If RunStatus > RunFinished Then
  Begin
  Action:=caMinimize;
  End
Else
  Begin
  While CollectionRoom.Count > 0 Do
    Begin
    Collection:=CollectionRoom.First;
    CollectionRoom.Remove(Collection);
    Collection.SaveToCSV(Collection.CSVFile,Collection.CTitle);
    Collection.Free;   {save to CSV included}
    End;
  Action:=caFree;
  End;
end;

procedure TGraphicsForm.GoHandleCSVEdit(Sender: TObject; var Key: Char);
Var
  ClickCollection: TomasCollection;
begin
If Key = #13 Then
  Begin
  With CollectionsGrid Do
    Begin
    ClickCollection:=CollectionRoom.Items[Row-1];
    ClickCollection.SetCSVFile(Trim(CSVEdit.Text));
    Cells[3,Row]:=Trim(CSVEdit.Text);
    CSVEdit.Hide;
    SetFocus;
    End;
  End;
If Key = #27 Then
  Begin
  CSVEdit.Hide;
  End;
end;

procedure TGraphicsForm.GoResizeGrid(Sender: TObject);
begin
  CollectionsGrid.Height := Height - 105;
  CollectionsGrid.Width := Width - 16;
end;

Initialization
  InitializeGraphics;

end.
